@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2002 Jean-Baptiste Mardelle <bj@altern.org>
- * Copyright (C) 2007,2008,2009,2010,2011,2012,2013,2016
+ * Copyright (C) 2007,2008,2009,2010,2011,2012,2013,2016,2017
  *               Rolf Eike Beer <kde@opensource.sf-tec.de>
  */
 /***************************************************************************
@@ -21,6 +21,7 @@
 #include "core/KGpgSubkeyNode.h"
 #include "core/KGpgUatNode.h"
 #include "core/KGpgUidNode.h"
+#include "kgpgsettings.h"
 
 #include <gpgme.h>
 #include <KLocalizedString>
@@ -34,10 +35,10 @@ using namespace KgpgCore;
 
 QString KgpgInterface::getGpgSetting(const QString &name, const QString &configfile)
 {
-	const QString tmp(name.simplified() + QLatin1Char( ' ' ));
+	const QString tmp = name.simplified() + QLatin1Char(' ');
 	QFile qfile(configfile);
 
-	if (qfile.open(QIODevice::ReadOnly) && (qfile.exists())) {
+	if (qfile.exists() && qfile.open(QIODevice::ReadOnly)) {
 		QTextStream t(&qfile);
 		while (!t.atEnd()) {
 			QString result(t.readLine().simplified());
@@ -56,8 +57,8 @@ void KgpgInterface::setGpgSetting(const QString &name, const QString &value, con
 {
 	QFile qfile(url);
 
-	if (qfile.open(QIODevice::ReadOnly) && (qfile.exists())) {
-		const QString temp(name + QLatin1Char( ' ' ));
+	if (qfile.exists() && qfile.open(QIODevice::ReadOnly)) {
+		const QString temp = name + QLatin1Char(' ');
 		QString texttowrite;
 		bool found = false;
 		QTextStream t(&qfile);
@@ -90,7 +91,7 @@ void KgpgInterface::setGpgSetting(const QString &name, const QString &value, con
 bool KgpgInterface::getGpgBoolSetting(const QString &name, const QString &configfile)
 {
 	QFile qfile(configfile);
-	if (qfile.open(QIODevice::ReadOnly) && (qfile.exists())) {
+	if (qfile.exists() && qfile.open(QIODevice::ReadOnly)) {
 		QTextStream t(&qfile);
 		while (!t.atEnd()) {
 			if (t.readLine().simplified().startsWith(name))
@@ -105,7 +106,7 @@ void KgpgInterface::setGpgBoolSetting(const QString &name, const bool enable, co
 {
 	QFile qfile(url);
 
-	if (qfile.open(QIODevice::ReadOnly) && (qfile.exists())) {
+	if (qfile.exists() && qfile.open(QIODevice::ReadOnly)) {
 		QString texttowrite;
 		bool found = false;
 		QTextStream t(&qfile);
@@ -147,10 +148,10 @@ readPublicKeysProcess(GPGProc &p, KGpgKeyNode *readNode)
 	QStringList lsp;
 	int items;
 	KgpgCore::KgpgKeyList publiclistkeys;
-	KgpgCore::KgpgKey *publickey = Q_NULLPTR;
+	KgpgCore::KgpgKey *publickey = nullptr;
 	unsigned int idIndex = 0;
 	QString log;
-	KGpgSignableNode *currentSNode = Q_NULLPTR;	///< the current (sub)node signatures are read for
+	KGpgSignableNode *currentSNode = nullptr;	///< the current (sub)node signatures are read for
 
 	while ((items = p.readln(lsp)) >= 0) {
 		if ((lsp.at(0) == QLatin1String( "pub" )) && (items >= 10)) {
@@ -183,11 +184,15 @@ readPublicKeysProcess(GPGProc &p, KGpgKeyNode *readNode)
 
 			publickey->setValid(enabled);  // disabled key
 
+			// GnuPG since 2.1 can flag immediately if a key has a secret key
+			if (items > 14 && lsp.at(14).contains(QLatin1Char('+')))
+				publickey->setSecret(true);
+
 			idIndex = 0;
 		} else if (publickey && (lsp.at(0) == QLatin1String("fpr")) && (items >= 10)) {
-			const QString fingervalue = lsp.at(9);
+			const QString &fingervalue = lsp.at(9);
 
-			if ((currentSNode != Q_NULLPTR) && (currentSNode->getType() == ITYPE_SUB))
+			if ((currentSNode != nullptr) && (currentSNode->getType() == ITYPE_SUB))
 				static_cast<KGpgSubkeyNode *>(currentSNode)->setFingerprint(fingervalue);
 			else if (publickey->fingerprint().isEmpty())
 				publickey->setFingerprint(fingervalue);
@@ -211,13 +216,13 @@ readPublicKeysProcess(GPGProc &p, KGpgKeyNode *readNode)
 			sub.setExpiration(Convert::toDateTime(lsp.at(6)));
 
 			publickey->subList()->append(sub);
-			if (readNode == Q_NULLPTR)
-				currentSNode = Q_NULLPTR;
+			if (readNode == nullptr)
+				currentSNode = nullptr;
 			else
 				currentSNode = new KGpgSubkeyNode(readNode, sub);
 		} else if (publickey && (lsp.at(0) == QLatin1String( "uat" ))) {
 			idIndex++;
-			if (readNode != Q_NULLPTR) {
+			if (readNode != nullptr) {
 				currentSNode = new KGpgUatNode(readNode, idIndex, lsp);
 			}
 		} else if (publickey && (lsp.at(0) == QLatin1String( "uid" )) && (items >= 10)) {
@@ -231,11 +236,11 @@ readPublicKeysProcess(GPGProc &p, KGpgKeyNode *readNode)
 						kmail = kmail.section(QLatin1Char( ')' ), 1);
 
 					kmail = kmail.section(QLatin1Char( '<' ), 1);
-					kmail.truncate(kmail.length() - 1);
+					kmail.chop(1);
 
 					if (kmail.contains(QLatin1Char( '<' ))) {
 						// several email addresses in the same key
-						kmail = kmail.replace(QLatin1Char( '>' ), QLatin1Char( ';' ));
+						kmail.replace(QLatin1Char( '>' ), QLatin1Char( ';' ));
 						kmail.remove(QLatin1Char( '<' ));
 					}
 				}
@@ -256,15 +261,15 @@ readPublicKeysProcess(GPGProc &p, KGpgKeyNode *readNode)
 				currentSNode = readNode;
 			} else {
 				idIndex++;
-				if (readNode != Q_NULLPTR) {
+				if (readNode != nullptr) {
 					currentSNode = new KGpgUidNode(readNode, idIndex, lsp);
 				}
 			}
 		} else if (publickey && ((lsp.at(0) == QLatin1String( "sig" )) || (lsp.at(0) == QLatin1String( "rev" ))) && (items >= 11)) {
 			// there are no strings here that could have a recoded QLatin1Char( ':' ) in them
-			const QString signature = lsp.join(QLatin1String(":"));
+			const QString signature = lsp.join(QLatin1Char(':'));
 
-			if (currentSNode != Q_NULLPTR)
+			if (currentSNode != nullptr)
 				(void) new KGpgSignNode(currentSNode, lsp);
 		} else {
 			log += lsp.join(QString(QLatin1Char( ':' ))) + QLatin1Char( '\n' );
@@ -272,7 +277,7 @@ readPublicKeysProcess(GPGProc &p, KGpgKeyNode *readNode)
 	}
 
 	if (p.exitCode() != 0) {
-		KMessageBox::detailedError(Q_NULLPTR, i18n("An error occurred while scanning your keyring"), log);
+		KMessageBox::detailedError(nullptr, i18n("An error occurred while scanning your keyring"), log);
 		log.clear();
 	}
 
@@ -286,14 +291,18 @@ KgpgKeyList KgpgInterface::readPublicKeys(const QStringList &ids)
 			QLatin1String("--with-colons") <<
 			QLatin1String("--with-fingerprint") <<
 			QLatin1String("--fixed-list-mode") <<
-			QLatin1String("--list-keys") <<
-			ids;
+			QLatin1String("--list-keys");
+
+	if (GPGProc::gpgVersion(GPGProc::gpgVersionString(KGpgSettings::gpgBinaryPath())) > 0x20100)
+		process << QLatin1String("--with-secret");
+
+	process << ids;
 
 	process.setOutputChannelMode(KProcess::MergedChannels);
 
 	process.start();
 	process.waitForFinished(-1);
-	return readPublicKeysProcess(process, Q_NULLPTR);
+	return readPublicKeysProcess(process, nullptr);
 }
 
 void KgpgInterface::readSignatures(KGpgKeyNode *node)
@@ -321,7 +330,7 @@ readSecretKeysProcess(GPGProc &p)
 	int items;
 	bool hasuid = false;
 	KgpgCore::KgpgKeyList result;
-	KgpgCore::KgpgKey *secretkey = Q_NULLPTR;
+	KgpgCore::KgpgKey *secretkey = nullptr;
 
 	while ( (items = p.readln(lsp)) >= 0 ) {
 		if ((lsp.at(0) == QLatin1String( "sec" )) && (items >= 10)) {
@@ -353,7 +362,7 @@ readSecretKeysProcess(GPGProc &p)
 
 			hasuid = true;
 
-			const QString fullname(lsp.at(9));
+			const QString &fullname = lsp.at(9);
 			if (fullname.contains(QLatin1Char( '<' ) )) {
 				QString kmail(fullname);
 
@@ -361,10 +370,10 @@ readSecretKeysProcess(GPGProc &p)
 					kmail = kmail.section(QLatin1Char( ')' ), 1);
 
 				kmail = kmail.section(QLatin1Char( '<' ), 1);
-				kmail.truncate(kmail.length() - 1);
+				kmail.chop(1);
 
 				if (kmail.contains(QLatin1Char( '<' ) )) { // several email addresses in the same key
-					kmail = kmail.replace(QLatin1Char( '>' ), QLatin1Char( ';' ));
+					kmail.replace(QLatin1Char( '>' ), QLatin1Char( ';' ));
 					kmail.remove(QLatin1Char( '<' ));
 				}
 
